@@ -199,6 +199,7 @@ from gdsfactory.components.rings import (
     disk,
     disk_heater,
     ring,
+    ring_asymmetric,
     ring_crow,
     ring_crow_couplers,
     ring_double,
@@ -545,6 +546,7 @@ __all__ = [
     "resistance_meander",
     "resistance_sheet",
     "ring",
+    "ring_asymmetric",
     "ring_crow",
     "ring_crow_couplers",
     "ring_double",
