@@ -5,35 +5,36 @@ from __future__ import annotations
 import importlib
 import pathlib
 import warnings
-from collections.abc import Callable
-from functools import cached_property, partial, wraps
-from typing import Any
+from collections.abc import Callable, Mapping, Sequence
+from functools import cached_property, partial
+from typing import Any, cast, overload
 
 import kfactory as kf
-import numpy as np
 import yaml
-from kfactory.kcell import LayerEnum
+from kfactory.layer import LayerEnum
 from pydantic import BaseModel, ConfigDict, Field
 
 from gdsfactory import logger
+from gdsfactory.component import Component, ComponentAllAngle
 from gdsfactory.config import CONF
+from gdsfactory.cross_section import CrossSection, Section
+from gdsfactory.cross_section import xsection as cross_section_xsection
 from gdsfactory.generic_tech import get_generic_pdk
 from gdsfactory.read.from_yaml_template import cell_from_yaml_template
-from gdsfactory.serialization import convert_tuples_to_lists
+from gdsfactory.serialization import clean_value_json, convert_tuples_to_lists
 from gdsfactory.symbols import floorplan_with_block_letters
 from gdsfactory.technology import LayerStack, LayerViews, klayout_tech
 from gdsfactory.typings import (
+    CellAllAngleSpec,
     CellSpec,
-    Component,
-    ComponentBase,
+    ComponentAllAngleFactory,
     ComponentFactory,
     ComponentSpec,
     ConnectivitySpec,
-    CrossSection,
     CrossSectionFactory,
     CrossSectionSpec,
-    Layer,
     LayerSpec,
+    LayerTransitions,
     MaterialSpec,
     PathType,
     RoutingStrategies,
@@ -62,7 +63,7 @@ def evanescent_coupler_sample() -> None:
     pass
 
 
-def extract_args_from_docstring(docstring: str) -> dict[str, Any] | None:
+def extract_args_from_docstring(docstring: str) -> dict[str, Any]:
     """This function extracts settings from a function's docstring for uPDK format.
 
     Args:
@@ -71,7 +72,7 @@ def extract_args_from_docstring(docstring: str) -> dict[str, Any] | None:
     Returns:
         settings (dict): The extracted YAML data as a dictionary.
     """
-    args_dict = {}
+    args_dict: dict[str, Any] = {}
 
     docstring_lines = docstring.split("\n")
     for line in docstring_lines:
@@ -117,6 +118,7 @@ class Pdk(BaseModel):
         version: PDK version.
         cross_sections: dict of cross_sections factories.
         cells: dict of parametric cells that return Components.
+        containers: dict of containers that return Components. A container is a cell that contains other cells.
         models: dict of models names to functions.
         symbols: dict of symbols names to functions.
         default_symbol_factory:
@@ -150,9 +152,10 @@ class Pdk(BaseModel):
         default_factory=dict, exclude=True
     )
     cells: dict[str, ComponentFactory] = Field(default_factory=dict, exclude=True)
-    models: dict[str, Callable] = Field(default_factory=dict, exclude=True)
+    containers: dict[str, ComponentFactory] = Field(default_factory=dict, exclude=True)
+    models: dict[str, Callable[..., Any]] = Field(default_factory=dict, exclude=True)
     symbols: dict[str, ComponentFactory] = Field(default_factory=dict)
-    default_symbol_factory: Callable[..., ComponentFactory] = Field(
+    default_symbol_factory: ComponentFactory = Field(
         default=floorplan_with_block_letters, exclude=True
     )
     base_pdks: list[Pdk] = Field(default_factory=list)
@@ -162,14 +165,12 @@ class Pdk(BaseModel):
     layers: type[LayerEnum] | None = None
     layer_stack: LayerStack | None = None
     layer_views: LayerViews | None = None
-    layer_transitions: dict[LayerSpec | tuple[Layer, Layer], ComponentSpec] = Field(
-        default_factory=dict
-    )
+    layer_transitions: LayerTransitions = Field(default_factory=dict)
     constants: dict[str, Any] = constants
     materials_index: dict[str, MaterialSpec] = Field(default_factory=dict)
     routing_strategies: RoutingStrategies | None = None
     bend_points_distance: float = 20 * nm
-    connectivity: list[ConnectivitySpec] | None = None
+    connectivity: Sequence[ConnectivitySpec] | None = None
     max_cellname_length: int = CONF.max_cellname_length
 
     model_config = ConfigDict(
@@ -185,27 +186,27 @@ class Pdk(BaseModel):
         Ensures that the cross-section name matches the name of the function
         that generated it when created using default parameters.
 
+        Reuses the core xsection decorator from cross_section.py while maintaining
+        PDK-specific storage of cross sections.
+
         .. code-block:: python
 
             @pdk.xsection
             def xs_sc(width=TECH.width_sc, radius=TECH.radius_sc):
                 return gf.cross_section.cross_section(width=width, radius=radius)
         """
-        default_xs = func()
-        self._cross_section_default_names[default_xs.name] = func.__name__
+        decorated_func = cross_section_xsection(func)
 
-        @wraps(func)
-        def newfunc(**kwargs: Any) -> CrossSection:
-            xs = func(**kwargs)
-            if xs.name in self._cross_section_default_names:
-                xs._name = self._cross_section_default_names[xs.name]
-            return xs
+        self.cross_sections[func.__name__] = decorated_func
 
-        self.cross_sections[func.__name__] = newfunc
-        return newfunc
+        return decorated_func
 
-    def activate(self) -> None:
+    def activate(self, force: bool = False) -> None:
         """Set current pdk to the active pdk (if not already active)."""
+        global _ACTIVE_PDK
+        if not force and _ACTIVE_PDK and _ACTIVE_PDK.name is self.name:
+            return
+
         logger.debug(f"{self.name!r} PDK {self.version} is now active")
 
         for pdk in self.base_pdks:
@@ -227,7 +228,7 @@ class Pdk(BaseModel):
                     "cells functions that return a Component"
                 )
             if name in self.cells:
-                warnings.warn(f"Overwriting cell {name!r}")
+                warnings.warn(f"Overwriting cell {name!r}", stacklevel=3)
 
             self.cells[name] = cell
 
@@ -240,7 +241,7 @@ class Pdk(BaseModel):
                     "cross_section functions that return a CrossSection"
                 )
             if name in self.cross_sections:
-                warnings.warn(f"Overwriting cross_section {name!r}")
+                warnings.warn(f"Overwriting cross_section {name!r}", stacklevel=3)
             self.cross_sections[name] = cross_section
 
     def register_cells_yaml(
@@ -290,20 +291,29 @@ class Pdk(BaseModel):
         self.cells.pop(name)
         logger.info(f"Removed cell {name!r}")
 
-    def get_cell(self, cell: CellSpec, **kwargs: Any) -> ComponentFactory:
+    @overload
+    def get_cell(self, cell: CellSpec, **kwargs: Any) -> ComponentFactory: ...
+    @overload
+    def get_cell(
+        self, cell: CellAllAngleSpec, **kwargs: Any
+    ) -> ComponentAllAngleFactory: ...
+
+    def get_cell(
+        self, cell: CellSpec | CellAllAngleSpec, **kwargs: Any
+    ) -> ComponentFactory | ComponentAllAngleFactory:
         """Returns ComponentFactory from a cell spec."""
-        cells = set(self.cells.keys())
+        cells_and_containers = self._get_cells_and_containers()
 
         if callable(cell):
             return cell
         elif isinstance(cell, str):
-            if cell not in cells:
-                matching_cells = [c for c in cells if cell in c]
+            if cell not in cells_and_containers:
+                matching_cells = [c for c in cells_and_containers if cell in c]
                 raise ValueError(
                     f"{cell!r} from PDK {self.name!r} not in cells: Did you mean {matching_cells}?"
                 )
-            return self.cells[cell]
-        elif isinstance(cell, dict):
+            return cells_and_containers[cell]
+        else:
             for key in cell.keys():
                 if key not in component_settings:
                     raise ValueError(
@@ -313,32 +323,42 @@ class Pdk(BaseModel):
             settings.update(**kwargs)
 
             cell_name = cell.get("function")
-            if not isinstance(cell_name, str) or cell_name not in cells:
-                cell_name = cell
-                matching_cells = [cell for cell in cells if cell_name in cell]
+            if not isinstance(cell_name, str) or cell_name not in cells_and_containers:
+                matching_cells = [c for c in cells_and_containers if c in cell.keys()]
                 raise ValueError(
                     f"{cell!r} from PDK {self.name!r} not in cells: Did you mean {matching_cells}?"
                 )
-            cell = self.cells[cell_name]
+            cell = cells_and_containers[cell_name]
             return partial(cell, **settings)
-        else:
-            raise ValueError(
-                "get_cell expects a CellSpec (ComponentFactory, string or dict),"
-                f"got {type(cell)}"
-            )
 
     def get_component(
         self,
         component: ComponentSpec,
-        settings: dict[str, Any] | None = None,
+        settings: Mapping[str, Any] | None = None,
+        include_containers: bool = True,
         **kwargs: Any,
     ) -> Component:
         """Returns component from a component spec."""
+        if include_containers:
+            cells = self._get_cells_and_containers()
+        else:
+            cells = self.cells
+
         return self._get_component(
-            component=component, cells=self.cells, settings=settings, **kwargs
+            component=component, cells=cells, settings=settings, **kwargs
         )
 
-    def get_symbol(self, component: ComponentSpec, **kwargs: Any) -> ComponentBase:
+    def _get_cells_and_containers(self) -> dict[str, ComponentFactory]:
+        """Returns a dictionary of cells and containers."""
+        cells_and_containers = {**self.cells, **self.containers}
+        conflicting_names = set(self.cells.keys()).intersection(self.containers.keys())
+        if conflicting_names:
+            raise ValueError(
+                f"PDK {self.name!r} has overlapping cell names between cells and containers: {list(conflicting_names)}. "
+            )
+        return cells_and_containers
+
+    def get_symbol(self, component: ComponentSpec, **kwargs: Any) -> Component:
         """Returns a component's symbol from a component spec."""
         # this is a pretty rough first implementation
         try:
@@ -352,8 +372,8 @@ class Pdk(BaseModel):
     def _get_component(
         self,
         component: ComponentSpec,
-        cells: dict[str, Callable[..., ComponentBase]],
-        settings: dict[str, Any] | None = None,
+        cells: dict[str, ComponentFactory],
+        settings: Mapping[str, Any] | None = None,
         **kwargs: Any,
     ) -> Component:
         """Returns component from a component spec.
@@ -363,22 +383,22 @@ class Pdk(BaseModel):
             cells: dict of cells.
             settings: settings to override.
             kwargs: settings to override.
-
         """
-        cells = sorted(cells)  # type: ignore
+        cell_names = sorted(cells)
 
         settings = settings or {}
         kwargs = kwargs or {}
         kwargs.update(settings)
 
-        if isinstance(component, ComponentBase):
-            return component  # type: ignore
-        elif isinstance(component, kf.KCell):
-            return Component.from_kcell(component)
+        if isinstance(component, kf.ProtoTKCell):
+            return Component(base=component.base)
+        elif isinstance(component, kf.VKCell):
+            return ComponentAllAngle(base=component.base)
         elif callable(component):
-            return component(**kwargs)
+            _component = component(**kwargs)
+            return type(_component)(base=_component.base)  # type: ignore[call-overload,no-any-return]
         elif isinstance(component, str):
-            if component not in cells:
+            if component not in cell_names:
                 substring = component
                 matching_cells: list[str] = []
 
@@ -391,8 +411,8 @@ class Pdk(BaseModel):
                 raise ValueError(
                     f"{component!r} not in PDK {self.name!r}. Did you mean {matching_cells}?"
                 )
-            return self.cells[component](**kwargs)
-        elif isinstance(component, dict):  # type: ignore
+            return cells[component](**kwargs)
+        elif isinstance(component, dict):
             for key in component.keys():
                 if key not in component_settings:
                     raise ValueError(
@@ -403,14 +423,14 @@ class Pdk(BaseModel):
 
             cell_name = component.get("component", None)
             cell_name = cell_name or component.get("function")
-            cell_name = cell_name.split(".")[-1]  # type: ignore
+            cell_name = str(cell_name).split(".")[-1]
 
-            if not isinstance(cell_name, str) or cell_name not in cells:
+            if cell_name not in cells:
                 matching_cells = [c for c in cells if cell_name in c]
                 raise ValueError(
                     f"{cell_name!r} from PDK {self.name!r} not in cells: Did you mean {matching_cells}?"
                 )
-            return self.cells[cell_name](**settings)
+            return cells[cell_name](**settings)
         else:
             raise ValueError(
                 "get_component expects a ComponentSpec (Component, ComponentFactory, "
@@ -436,44 +456,67 @@ class Pdk(BaseModel):
             return xs(**kwargs)
         elif isinstance(cross_section, dict):
             xs_name = cross_section.get("cross_section", None)
+            if xs_name is None:
+                raise ValueError("cross_section name is required")
             settings = cross_section.get("settings", {})
             return self.get_cross_section(xs_name, **settings)
         elif isinstance(cross_section, CrossSection):
             if kwargs:
                 warnings.warn(
-                    f"{kwargs} are ignored for cross_section {cross_section.name!r}"
+                    f"{kwargs} ignored for cross_section {cross_section.name!r}",
+                    stacklevel=3,
                 )
+
             return cross_section
+        elif isinstance(cross_section, kf.DCrossSection | kf.SymmetricalCrossSection):
+            from gdsfactory import kcl
+
+            if isinstance(cross_section, kf.DCrossSection):
+                cross_section_ = cross_section.base
+            else:
+                cross_section_ = cross_section
+            section_ = Section(
+                width=kcl.to_um(cross_section_.width),
+                layer=kcl.layout.layer(cross_section_.main_layer),
+            )
+            xs_ = CrossSection(
+                sections=(section_,),
+                radius=kcl.to_um(cross_section_.radius),
+                radius_min=kcl.to_um(cross_section_.radius_min),
+            )
+            xs_._name = cross_section_.name
+            return xs_
         else:
             raise ValueError(
                 "get_cross_section expects a CrossSectionSpec (CrossSection, "
                 f"CrossSectionFactory, Transition, string or dict), got {type(cross_section)}"
             )
 
-    def get_layer(self, layer: LayerSpec) -> LayerEnum:
+    def get_layer(self, layer: LayerSpec | kf.kdb.LayerInfo) -> LayerEnum | int:
         """Returns layer from a layer spec."""
-        if isinstance(layer, LayerEnum):
+        if isinstance(layer, LayerEnum | int):
             return layer
         elif isinstance(layer, tuple | list):
             if len(layer) != 2:
                 raise ValueError(f"{layer!r} needs two integer numbers.")
-            return kf.kcl.layer(*layer)
-        elif isinstance(layer, str):
+            return kf.kcl.layout.layer(*layer)
+        elif isinstance(layer, kf.kdb.LayerInfo):
+            return layer.layer
+        else:
             if not hasattr(self.layers, layer):
                 raise ValueError(f"{layer!r} not in {self.layers}")
-            return getattr(self.layers, layer)
-        elif isinstance(layer, int):
-            return layer
-        elif layer is np.nan:
-            return np.nan
-        else:
-            raise ValueError(
-                f"{layer!r} needs to be a LayerSpec (string, int or (int, int) or LayerEnum), got {type(layer)}"
-            )
+            return cast(LayerEnum, getattr(self.layers, layer))
 
     def get_layer_name(self, layer: LayerSpec) -> str:
         layer_index = self.get_layer(layer)
-        return self.layers[layer_index]
+        assert self.layers is not None
+        try:
+            return str(self.layers[layer_index])  # type: ignore[index]
+        except Exception:
+            try:
+                return str(self.layers(layer_index))  # type: ignore[call-arg]
+            except Exception:
+                raise ValueError(f"Could not find name for layer {layer_index}")
 
     def get_layer_views(self) -> LayerViews:
         if self.layer_views is None:
@@ -486,39 +529,39 @@ class Pdk(BaseModel):
         return self.layer_stack
 
     def get_constant(self, key: str) -> Any:
-        if not isinstance(key, str):
-            return key
         if key not in self.constants:
             constants = list(self.constants.keys())
             raise ValueError(f"{key!r} not in {constants}")
         return self.constants[key]
 
-    def to_updk(self) -> str:
+    def to_updk(self, exclude: Sequence[str] | None = None) -> str:
         """Export to uPDK YAML definition."""
-        from gdsfactory.components.bbox import bbox_to_points
+        from gdsfactory.components import bbox_to_points
 
-        blocks = {cell_name: cell() for cell_name, cell in self.cells.items()}
-        blocks = {
-            name: dict(
+        exclude = exclude or []
+        _blocks = {
+            cell_name: cell()
+            for cell_name, cell in self.cells.items()
+            if cell_name not in exclude
+        }
+        blocks: dict[str, dict[str, Any]] = {}
+        for name, c in _blocks.items():
+            if c.__doc__ is None:
+                continue
+            extra_args = extract_args_from_docstring(c.__doc__)
+
+            blocks[name] = dict(
                 bbox=bbox_to_points(c.dbbox()),
                 doc=c.__doc__.split("\n")[0],
-                settings=extract_args_from_docstring(c.__doc__),
+                settings=extra_args,
                 parameters={
                     sname: {
-                        "value": svalue,
+                        "value": clean_value_json(svalue),
                         "type": str(svalue.__class__.__name__),
-                        "doc": extract_args_from_docstring(c.__doc__)
-                        .get(sname, {})
-                        .get("doc", None),
-                        "min": extract_args_from_docstring(c.__doc__)
-                        .get(sname, {})
-                        .get("min", 0),
-                        "max": extract_args_from_docstring(c.__doc__)
-                        .get(sname, {})
-                        .get("max", 0),
-                        "unit": extract_args_from_docstring(c.__doc__)
-                        .get(sname, {})
-                        .get("unit", None),
+                        "doc": extra_args.get(sname, {}).get("doc", None),
+                        "min": extra_args.get(sname, {}).get("min", 0),
+                        "max": extra_args.get(sname, {}).get("max", 0),
+                        "unit": extra_args.get(sname, {}).get("unit", None),
                     }
                     for sname, svalue in c.settings
                     if isinstance(svalue, str | float | int)
@@ -530,8 +573,8 @@ class Pdk(BaseModel):
                         if hasattr(port, "cross_section")
                         else "",
                         "xya": [
-                            float(port.dcenter[0]),
-                            float(port.dcenter[1]),
+                            float(port.center[0]),
+                            float(port.center[1]),
                             float(port.orientation),
                         ],
                         "alias": port.info.get("alias"),
@@ -540,29 +583,26 @@ class Pdk(BaseModel):
                     for port in c.ports
                 },
             )
-            for name, c in blocks.items()
-        }
         xsections = {
             xs_name: self.get_cross_section(xs_name)
             for xs_name in self.cross_sections.keys()
         }
-        xsections = {
+        xsections_widths = {
             xs_name: dict(width=xsection.width)
             for xs_name, xsection in xsections.items()
         }
 
         header = dict(description=self.name)
 
-        d = {"blocks": blocks, "xsections": xsections, "header": header}
-        d = convert_tuples_to_lists(d)
-        return yaml.dump(d)
+        d = {"blocks": blocks, "xsections": xsections_widths, "header": header}
+        return yaml.dump(convert_tuples_to_lists(d))
 
     def get_cross_section_name(self, cross_section: CrossSection) -> str:
         xs_name = next(
             (
                 key
                 for key, value in self.cross_sections.items()
-                if value == cross_section
+                if value() == cross_section
             ),
             None,
         )
@@ -580,7 +620,7 @@ class Pdk(BaseModel):
                 name=self.name,
                 layer_views=self.layer_views,
                 connectivity=self.connectivity,
-                layer_map=self.layers,
+                layer_map=self.layers,  # type: ignore[arg-type]
                 layer_stack=self.layer_stack,
             )
         except AttributeError as e:
@@ -608,6 +648,7 @@ def get_active_pdk(name: str | None = None) -> Pdk:
 
         else:
             raise ValueError("no active pdk")
+    assert _ACTIVE_PDK is not None, "Could not find active PDK"
     return _ACTIVE_PDK
 
 
@@ -617,16 +658,24 @@ def get_material_index(material: MaterialSpec, *args: Any, **kwargs: Any) -> Com
         raise NotImplementedError(
             "The active PDK does not implement 'get_material_index'"
         )
-    return active_pdk.get_material_index(material, *args, **kwargs)
+    return active_pdk.get_material_index(material, *args, **kwargs)  # type: ignore[no-any-return]
 
 
 def get_component(
-    component: ComponentSpec, settings: dict[str, Any] | None = None, **kwargs: Any
+    component: ComponentSpec, settings: Mapping[str, Any] | None = None, **kwargs: Any
 ) -> Component:
     return get_active_pdk().get_component(component, settings=settings, **kwargs)
 
 
-def get_cell(cell: CellSpec, **kwargs: Any) -> ComponentFactory:
+@overload
+def get_cell(cell: CellSpec, **kwargs: Any) -> ComponentFactory: ...
+@overload
+def get_cell(cell: CellAllAngleSpec, **kwargs: Any) -> ComponentAllAngleFactory: ...
+
+
+def get_cell(
+    cell: CellSpec | CellAllAngleSpec, **kwargs: Any
+) -> ComponentFactory | ComponentAllAngleFactory:
     return get_active_pdk().get_cell(cell, **kwargs)
 
 
@@ -634,13 +683,12 @@ def get_cross_section(cross_section: CrossSectionSpec, **kwargs: Any) -> CrossSe
     return get_active_pdk().get_cross_section(cross_section, **kwargs)
 
 
-def get_layer(layer: LayerSpec) -> LayerEnum:
+def get_layer(layer: LayerSpec | kf.kdb.LayerInfo) -> LayerEnum | int:
     return get_active_pdk().get_layer(layer)
 
 
 def get_layer_name(layer: LayerSpec) -> str:
-    layer_index = get_layer(layer)
-    return str(get_active_pdk().layers(layer_index))  # type: ignore
+    return get_active_pdk().get_layer_name(layer)
 
 
 def get_layer_tuple(layer: LayerSpec) -> tuple[int, int]:
@@ -687,26 +735,32 @@ def get_routing_strategies() -> RoutingStrategies:
 if __name__ == "__main__":
     import gdsfactory as gf
 
-    sample_mirror = """
-name: sample_mirror
+    sample_routing_sbend = """
 instances:
-    mmi1:
-      component: mmi1x2
+    cp1:
+      component: coupler
 
-    mmi2:
-      component: mmi1x2
+    cp2:
+      component: coupler
 
 placements:
-    mmi1:
-        xmax: 0
+    cp1:
+        x: 0
 
-    mmi2:
-        xmin: mmi1,east
-        mirror: True
+    cp2:
+        x: 300
+        y: 300
+
+routes:
+    bundle1:
+        links:
+          cp1,o3: cp2,o2
+        routing_strategy: route_bundle_sbend
 
 """
 
-    c = gf.read.from_yaml(sample_mirror)
+    c = gf.read.from_yaml(sample_routing_sbend)
+    c.show()
 
     # l1 = get_layer((1, 0))
     # l2 = get_layer((3, 0))
