@@ -1,10 +1,7 @@
 from __future__ import annotations
 
-from functools import partial
-
 import gdsfactory as gf
 from gdsfactory.component import Component
-from gdsfactory.components.wire import wire_straight
 from gdsfactory.port import select_ports_electrical
 from gdsfactory.routing.route_quad import route_quad
 from gdsfactory.typings import (
@@ -16,15 +13,12 @@ from gdsfactory.typings import (
     Strs,
 )
 
-_wire_long = partial(wire_straight, length=200.0)
 
-
-@gf.cell
 def add_electrical_pads_shortest(
-    component: ComponentSpec = _wire_long,
+    component: ComponentSpec = "wire_straight",
     pad: ComponentSpec = "pad",
     pad_port_spacing: float = 50.0,
-    pad_size: Size = (100.0, 100.0),
+    pad_size: Size | None = None,
     select_ports: PortsFactory = select_ports_electrical,
     port_names: Strs | None = None,
     port_orientation: AngleInDegrees = 90,
@@ -46,17 +40,19 @@ def add_electrical_pads_shortest(
         :include-source:
 
         import gdsfactory as gf
-        c = gf.components.straight_heater_metal(length=100)
-        wire_long = gf.components.wire_straight(length=200.)
-        c = gf.routing.add_electrical_pads_shortest(wire_long)
+        c = gf.components.cross(length=100, layer=(49, 0), port_type="electrical")
+        c = gf.routing.add_electrical_pads_shortest(c, pad_port_spacing=200)
         c.plot()
 
     """
     c = Component()
     component = gf.get_component(component)
-    pad = gf.get_component(pad, size=pad_size)
 
-    c.component = component
+    if pad_size is not None:
+        pad = gf.get_component(pad, size=pad_size)
+    else:
+        pad = gf.get_component(pad)
+
     ref = c << component
     ports = (
         [ref[port_name] for port_name in port_names]
@@ -69,20 +65,20 @@ def add_electrical_pads_shortest(
         port_orientation = port.orientation
 
         if port_orientation == 0:
-            p.dx = port.dx + pad_port_spacing
-            p.dy = port.dy
+            p.x = port.x + pad_port_spacing
+            p.y = port.y
             route_quad(component=c, port1=port, port2=p.ports["e1"], layer=layer)
         elif port_orientation == 180:
-            p.dx = port.dx - pad_port_spacing
-            p.dy = port.dy
+            p.x = port.x - pad_port_spacing
+            p.y = port.y
             route_quad(c, port, p.ports["e3"], layer=layer)
         elif port_orientation == 90:
-            p.dy = port.dy + pad_port_spacing
-            p.dx = port.dx
+            p.y = port.y + pad_port_spacing
+            p.x = port.x
             route_quad(c, port, p.ports["e4"], layer=layer)
         elif port_orientation == 270:
-            p.dy = port.dy - pad_port_spacing
-            p.dx = port.dx
+            p.y = port.y - pad_port_spacing
+            p.x = port.x
             route_quad(c, port, p.ports["e2"], layer=layer)
 
         c.add_port(port=p.ports["pad"], name=f"elec-{component.name}-{i + 1}")
