@@ -1,25 +1,20 @@
 from __future__ import annotations
 
-from functools import partial
-from typing import Any
+from typing import Any, Literal
 
 import gdsfactory as gf
 from gdsfactory.component import Component
-from gdsfactory.components.pad import pad_array as pad_array_function
-from gdsfactory.components.wire import wire_straight
 from gdsfactory.port import select_ports_electrical
 from gdsfactory.routing.route_quad import route_quad
 from gdsfactory.routing.sort_ports import sort_ports_x
 from gdsfactory.typings import ComponentSpec, Float2, LayerSpec, SelectPorts, Strs
 
-_wire_long = partial(wire_straight, length=200.0)
-
 
 def add_electrical_pads_top(
-    component: ComponentSpec = _wire_long,
-    direction: str = "top",
+    component: ComponentSpec,
+    direction: Literal["top", "right"] = "top",
     spacing: Float2 = (0.0, 100.0),
-    pad_array: ComponentSpec = pad_array_function,
+    pad_array: ComponentSpec = "pad_array",
     select_ports: SelectPorts = select_ports_electrical,
     port_names: Strs | None = None,
     layer: LayerSpec = "MTOP",
@@ -76,8 +71,11 @@ def add_electrical_pads_top(
         pads = c << gf.get_component(
             pad_array, columns=1, rows=len(ports_electrical), orientation=270
         )
-    pads.dx = ref.dx + spacing[0]
-    pads.dymin = ref.dymax + spacing[1]
+    else:
+        raise ValueError(f"Invalid direction {direction}")
+
+    pads.x = ref.x + spacing[0]
+    pads.ymin = ref.ymax + spacing[1]
 
     ports_pads = sort_ports_x(pads.ports)
     ports_component = sort_ports_x(ports_electrical)
@@ -96,9 +94,9 @@ def add_electrical_pads_top(
 
 
 if __name__ == "__main__":
-    import gdsfactory as gf
+    from gdsfactory.components import straight_heater_metal
 
-    c = gf.components.straight_heater_metal()
+    c = straight_heater_metal()
     # c = gf.components.mzi_phase_shifter_top_heater_metal()
     # cc = gf.routing.add_electrical_pads_top(component=c, spacing=(-150, 30))
     c = add_electrical_pads_top(c)
