@@ -1,13 +1,15 @@
 from __future__ import annotations
 
+from collections.abc import Sequence
+from functools import partial
+
 import numpy as np
 from kfactory.routing.generic import ManhattanRoute
 
 import gdsfactory as gf
+from gdsfactory import typings
 from gdsfactory.component import Component, ComponentReference
-from gdsfactory.components.bend_euler import bend_euler
-from gdsfactory.components.straight import straight as straight_function
-from gdsfactory.cross_section import CrossSection, strip
+from gdsfactory.cross_section import CrossSection
 from gdsfactory.port import Port, select_ports_optical
 from gdsfactory.routing.auto_taper import add_auto_tapers
 from gdsfactory.routing.route_single import route_single
@@ -25,15 +27,15 @@ def route_south(
     component: Component,
     component_to_route: Component | ComponentReference,
     optical_routing_type: int = 1,
-    excluded_ports: tuple[str, ...] | None = None,
+    excluded_ports: Sequence[str] | None = None,
     straight_separation: float = 4.0,
     io_gratings_lines: list[list[ComponentReference]] | None = None,
     gc_port_name: str = "o1",
-    bend: ComponentSpec = bend_euler,
-    straight: ComponentSpec = straight_function,
+    bend: ComponentSpec = "bend_euler",
+    straight: ComponentSpec = "straight",
     select_ports: SelectPorts = select_ports_optical,
     port_names: Strs | None = None,
-    cross_section: CrossSectionSpec = strip,
+    cross_section: CrossSectionSpec = "strip",
     start_straight_length: float = 0.5,
     port_type: str | None = None,
     allow_width_mismatch: bool = False,
@@ -79,29 +81,30 @@ def route_south(
         c.plot()
 
     """
-    c = component
-    component = component_to_route
     xs = gf.get_cross_section(cross_section)
     excluded_ports = excluded_ports or ()
     start_straight_length0 = start_straight_length
-    routes = []
+    routes: list[ManhattanRoute] = []
 
-    if optical_routing_type not in {
-        1,
-        2,
-    }:
+    if optical_routing_type not in {1, 2}:
         raise ValueError(
             f"optical_routing_type = {optical_routing_type} not in supported [1, 2]"
         )
 
     if port_names:
-        optical_ports = [component[port_name] for port_name in port_names]
+        optical_ports = [component_to_route[port_name] for port_name in port_names]
     else:
-        optical_ports = select_ports(component.ports)
-        optical_ports = [p for p in optical_ports if p.name not in excluded_ports]
+        optical_ports = [
+            p
+            for p in select_ports(component_to_route.ports)
+            if p.name not in excluded_ports
+        ]
 
     if auto_taper:
-        optical_ports = add_auto_tapers(c, optical_ports, cross_section)
+        optical_ports = add_auto_tapers(component, optical_ports, cross_section)
+
+    if not optical_ports:
+        return []
 
     port_type = port_type or optical_ports[0].port_type
     bend90 = bend(cross_section=cross_section) if callable(bend) else bend
@@ -109,10 +112,9 @@ def route_south(
     dy = abs(bend90.info["dy"])
 
     # Handle empty list gracefully
-    if not optical_ports:
-        return [], []
 
-    conn_params = dict(
+    route_single_with_conn_params = partial(
+        route_single,
         bend=bend,
         straight=straight,
         cross_section=cross_section,
@@ -145,26 +147,26 @@ def route_south(
         x: float, component_references: list[ComponentReference]
     ) -> np.intp:
         return np.array(
-            [abs(x - p.ports[gc_port_name].dx) for p in component_references]
+            [abs(x - p.ports[gc_port_name].x) for p in component_references]
         ).argmin()
 
     def gen_port_from_port(
-        x: float, y: float, p: Port, cross_section: CrossSection
+        x: float, y: float, p: typings.Port, cross_section: CrossSection
     ) -> Port:
         return Port(
             name=p.name,
             center=(x, y),
             orientation=90.0,
-            width=p.dwidth,
-            layer=cross_section.layer,
+            width=p.width,
+            layer=gf.get_layer(cross_section.layer),
             port_type=p.port_type,
         )
 
     west_ports.reverse()
-    y0 = min(p.dy for p in ordered_ports) - dy - 0.5
-    ports_to_route = []
+    y0 = min(p.y for p in ordered_ports) - dy - 0.5
+    ports_to_route: list[Port] = []
 
-    optical_xs_tmp = [p.dx for p in ordered_ports]
+    optical_xs_tmp = [p.x for p in ordered_ports]
     x_optical_min = min(optical_xs_tmp)
     x_optical_max = max(optical_xs_tmp)
 
@@ -175,14 +177,10 @@ def route_south(
     # The starting x depends on the heuristic chosen : ``1`` or ``2``
     if optical_routing_type == 1:
         # use component size to know how far to route
-        x = component.dxmin - dy - 1
-    elif optical_routing_type == 2:
+        x = component_to_route.xmin - dy - 1
+    else:
         # use optical port to know how far to route
         x = x_optical_min - dy - 1
-    else:
-        raise ValueError(
-            f"Invalid optical routing type {optical_routing_type!r} not in [1, 2]"
-        )
 
     # First route the ports facing west
     # In case we have to connect these ports to a line of gratings,
@@ -191,7 +189,7 @@ def route_south(
     for p in west_ports:
         if io_gratings_lines:
             i_grating = get_index_port_closest_to_x(x, io_gratings_lines[-1])
-            x_gr = io_gratings_lines[-1][i_grating].ports[gc_port_name].dx
+            x_gr = io_gratings_lines[-1][i_grating].ports[gc_port_name].x
             if abs(x - x_gr) < delta_gr_min:
                 if x > x_gr:
                     x = x_gr
@@ -200,7 +198,7 @@ def route_south(
 
         tmp_port = gen_port_from_port(x, y0, p, cross_section=xs)
         ports_to_route.append(tmp_port)
-        route = route_single(c, tmp_port, p, **conn_params)
+        route = route_single_with_conn_params(component, tmp_port, p)
         x -= sep
 
     # route first halft of north ports above the top west one
@@ -208,15 +206,14 @@ def route_south(
 
     start_straight_length = start_straight_length0
     if len(north_start) > 0:
-        y_max = max(p.dy for p in west_ports + north_start)
+        y_max = max(p.y for p in west_ports + north_start)
         for p in north_start:
             tmp_port = gen_port_from_port(x, y0, p, cross_section=xs)
-            route = route_single(
-                component=c,
+            route = route_single_with_conn_params(
+                component,
                 port2=p,
                 port1=tmp_port,
-                end_straight_length=start_straight_length + y_max - p.dy,
-                **conn_params,
+                end_straight_length=start_straight_length + y_max - p.y,
             )
 
             ports_to_route.append(tmp_port)
@@ -227,14 +224,10 @@ def route_south(
     # Set starting ``x`` on the east side
     if optical_routing_type == 1:
         #  use component size to know how far to route
-        x = component.dxmax + dy + 1
+        x = component_to_route.xmax + dy + 1
     elif optical_routing_type == 2:
         # use optical port to know how far to route
         x = x_optical_max + dy + 1
-    else:
-        raise ValueError(
-            f"Invalid optical routing type. Got {optical_routing_type}, only (1, 2 supported) "
-        )
 
     # Route the east ports
     # In case we have to connect these ports to a line of gratings,
@@ -244,7 +237,7 @@ def route_south(
     for p in east_ports:
         if io_gratings_lines:
             i_grating = get_index_port_closest_to_x(x, io_gratings_lines[-1])
-            x_gr = io_gratings_lines[-1][i_grating].ports[gc_port_name].dx
+            x_gr = io_gratings_lines[-1][i_grating].ports[gc_port_name].x
             if abs(x - x_gr) < delta_gr_min:
                 if x < x_gr:
                     x = x_gr
@@ -252,12 +245,11 @@ def route_south(
                     x = x_gr + delta_gr_min
 
         tmp_port = gen_port_from_port(x, y0, p, cross_section=xs)
-        route = route_single(
-            c,
+        route = route_single_with_conn_params(
+            component,
             tmp_port,
             p,
             end_straight_length=start_straight_length,
-            **conn_params,
         )
         routes.append(route)
         ports_to_route.append(tmp_port)
@@ -266,16 +258,15 @@ def route_south(
     # Route the remaining north ports
     start_straight_length = start_straight_length0
     if len(north_finish) > 0:
-        y_max = max(p.dy for p in east_ports + north_finish)
+        y_max = max(p.y for p in east_ports + north_finish)
         for p in north_finish:
             tmp_port = gen_port_from_port(x, y0, p, cross_section=xs)
             ports_to_route.append(tmp_port)
-            route = route_single(
-                c,
+            route = route_single_with_conn_params(
+                component,
                 tmp_port,
                 p,
-                end_straight_length=start_straight_length + y_max - p.dy,
-                **conn_params,
+                end_straight_length=start_straight_length + y_max - p.y,
             )
             x += sep
             start_straight_length += sep
@@ -284,9 +275,9 @@ def route_south(
     flipped_ports = [p.copy() for p in ports_to_route]
     for p in flipped_ports:
         p.trans *= gf.kdb.Trans.R180
-    c.add_ports(flipped_ports)
-    c.add_ports(south_ports)
-    c.auto_rename_ports()
+    component.add_ports(flipped_ports)
+    component.add_ports(south_ports)
+    component.auto_rename_ports()
     return routes
 
 
