@@ -1,17 +1,19 @@
 from __future__ import annotations
 
-from numpy import float64
+from collections.abc import Sequence
 
-from gdsfactory.port import Port
+from gdsfactory.typings import Port
 
 
 class RouteWarning(UserWarning):
     pass
 
 
-def direction_ports_from_list_ports(optical_ports: list[Port]) -> dict[str, list[Port]]:
+def direction_ports_from_list_ports(
+    optical_ports: Sequence[Port],
+) -> dict[str, list[Port]]:
     """Returns a dict of WENS ports."""
-    direction_ports = {x: [] for x in ["E", "N", "W", "S"]}
+    direction_ports: dict[str, list[Port]] = {x: [] for x in ["E", "N", "W", "S"]}
     for p in optical_ports:
         orientation = (p.orientation + 360.0) % 360
         if orientation <= 45.0 or orientation >= 315:
@@ -25,15 +27,15 @@ def direction_ports_from_list_ports(optical_ports: list[Port]) -> dict[str, list
 
     for direction, list_ports in list(direction_ports.items()):
         if direction in ["E", "W"]:
-            list_ports.sort(key=lambda p: p.dy)
+            list_ports.sort(key=lambda p: p.y)
 
         if direction in ["S", "N"]:
-            list_ports.sort(key=lambda p: p.dx)
+            list_ports.sort(key=lambda p: p.x)
 
     return direction_ports
 
 
-def check_ports_have_equal_spacing(list_ports: list[Port]) -> float64:
+def check_ports_have_equal_spacing(list_ports: Sequence[Port]) -> float:
     """Returns port separation.
 
     Raises error if not constant.
@@ -46,19 +48,18 @@ def check_ports_have_equal_spacing(list_ports: list[Port]) -> float64:
 
     orientation = get_list_ports_angle(list_ports)
     if orientation in [0, 180]:
-        xys = [p.dy for p in list_ports]
+        xys = [p.y for p in list_ports]
     else:
-        xys = [p.dx for p in list_ports]
+        xys = [p.x for p in list_ports]
 
     seps = [round(abs(c2 - c1), 5) for c1, c2 in zip(xys[1:], xys[:-1])]
     different_seps = set(seps)
     if len(different_seps) > 1:
-        raise ValueError("Ports should have the same separation. Got {different_seps}")
-
-    return different_seps.pop()
+        raise ValueError(f"Ports should have the same separation. Got {different_seps}")
+    return float(seps[0])
 
 
-def get_list_ports_angle(list_ports: list[Port]) -> float64 | int:
+def get_list_ports_angle(list_ports: Sequence[Port]) -> float | None:
     """Returns the orientation/angle (in degrees) of a list of ports."""
     if not list_ports:
         return None
@@ -68,8 +69,8 @@ def get_list_ports_angle(list_ports: list[Port]) -> float64 | int:
 
 
 if __name__ == "__main__":
-    import gdsfactory as gf
+    from gdsfactory.components import mmi1x2
 
-    c = gf.components.mmi1x2()
+    c = mmi1x2()
     d = direction_ports_from_list_ports(c.get_ports_list())
     c.show()
