@@ -6,7 +6,7 @@ import gdsfactory as gf
 
 
 @gf.cell
-def mzi_with_bend(radius: float = 10):
+def mzi_with_bend(radius: float = 10) -> gf.Component:
     """Returns MZI interferometer with bend."""
     c = gf.Component()
     mzi = c.add_ref(gf.components.mzi())
@@ -18,6 +18,6 @@ def mzi_with_bend(radius: float = 10):
 
 
 if __name__ == "__main__":
-    c = mzi_with_bend(radius=25)
+    c = mzi_with_bend(radius=50)
     # c = gf.routing.add_fiber_single(c)
-    c.show(show_ports=True)
+    c.show()
