@@ -8,7 +8,7 @@ if __name__ == "__main__":
     xsize = ysize = 100
     for i in range(1, 100):
         r = c << gf.components.mzi(delta_length=i)
-        r.dmove((i % 10 * xsize, i // 10 * ysize))
+        r.move((i % 10 * xsize, i // 10 * ysize))
 
     print(f"Elapsed time: {time.time() - time_start:.2f} s")
     c.show()
