@@ -7,17 +7,17 @@ You can maintain LayerViews in YAML (.yaml) or Klayout XML file (.lyp)
 from __future__ import annotations
 
 import builtins
-import os
 import pathlib
 import re
 import warnings
 import xml.etree.ElementTree as ET
-from typing import TYPE_CHECKING, Any
+from collections.abc import Mapping
+from typing import TYPE_CHECKING, Any, TypeAlias
 
 import numpy as np
 import yaml
 from kfactory import logger
-from kfactory.kcell import LayerEnum
+from kfactory.layer import LayerEnum
 from pydantic import BaseModel, ConfigDict, Field, field_validator
 from pydantic.color import ColorType
 from pydantic_extra_types.color import Color
@@ -25,19 +25,16 @@ from pydantic_extra_types.color import Color
 from gdsfactory.name import clean_name
 from gdsfactory.technology.color_utils import ensure_six_digit_hex_color
 from gdsfactory.technology.xml_utils import make_pretty_xml
-from gdsfactory.technology.yaml_utils import (
-    add_color_yaml_presenter,
-    add_multiline_str_yaml_presenter,
-    add_tuple_yaml_presenter,
-)
+from gdsfactory.technology.yaml_utils import TechnologyDumper
 
 if TYPE_CHECKING:
-    from pydantic.typing import AbstractSetIntStr, DictStrAny, MappingIntStrAny
-
     from gdsfactory.component import Component
 
 PathLike = pathlib.Path | str
 Layer = tuple[int, int]
+IncEx: TypeAlias = (
+    set[int] | set[str] | Mapping[int, "IncEx | bool"] | Mapping[str, "IncEx | bool"]
+)
 
 _klayout_line_styles = {
     "solid": "",
@@ -52,9 +49,9 @@ _klayout_line_styles = {
 _klayout_dither_patterns = {
     "solid": "*",
     "hollow": ".",
-    "dotted": "*.\n" ".*",
-    "coarsely dotted": "*...\n" "....\n" "..*.\n" "....",
-    "left-hatched": "*...\n" ".*..\n" "..*.\n" "...*",
+    "dotted": "*.\n.*",
+    "coarsely dotted": "*...\n....\n..*.\n....",
+    "left-hatched": "*...\n.*..\n..*.\n...*",
     "lightly left-hatched": "*.......\n"
     ".*......\n"
     "..*.....\n"
@@ -63,7 +60,7 @@ _klayout_dither_patterns = {
     ".....*..\n"
     "......*.\n"
     ".......*",
-    "strongly left-hatched dense": "**..\n" ".**.\n" "..**\n" "*..*",
+    "strongly left-hatched dense": "**..\n.**.\n..**\n*..*",
     "strongly left-hatched sparse": "**......\n"
     ".**.....\n"
     "..**....\n"
@@ -72,7 +69,7 @@ _klayout_dither_patterns = {
     ".....**.\n"
     "......**\n"
     "*......*",
-    "right-hatched": "*...\n" "...*\n" "..*.\n" ".*..",
+    "right-hatched": "*...\n...*\n..*.\n.*..",
     "lightly right-hatched": "*.......\n"
     ".......*\n"
     "......*.\n"
@@ -81,7 +78,7 @@ _klayout_dither_patterns = {
     "...*....\n"
     "..*.....\n"
     ".*......",
-    "strongly right-hatched dense": "**..\n" "*..*\n" "..**\n" ".**.",
+    "strongly right-hatched dense": "**..\n*..*\n..**\n.**.",
     "strongly right-hatched sparse": "**......\n"
     "*......*\n"
     "......**\n"
@@ -90,7 +87,7 @@ _klayout_dither_patterns = {
     "...**...\n"
     "..**....\n"
     ".**.....",
-    "cross-hatched": "*...\n" ".*.*\n" "..*.\n" ".*.*",
+    "cross-hatched": "*...\n.*.*\n..*.\n.*.*",
     "lightly cross-hatched": "*.......\n"
     ".*.....*\n"
     "..*...*.\n"
@@ -99,7 +96,7 @@ _klayout_dither_patterns = {
     "...*.*..\n"
     "..*...*.\n"
     ".*.....*",
-    "checkerboard 2px": "**..\n" "**..\n" "..**\n" "..**",
+    "checkerboard 2px": "**..\n**..\n..**\n..**",
     "strongly cross-hatched sparse": "**......\n"
     "***....*\n"
     "..**..**\n"
@@ -236,14 +233,14 @@ _klayout_dither_patterns = {
     "..*...*.\n"
     "*....*..\n"
     ".*.*....",
-    "vertical dense": "*.\n" "*.\n",
-    "vertical": ".*..\n" ".*..\n" ".*..\n" ".*..\n",
-    "vertical thick": ".**.\n" ".**.\n" ".**.\n" ".**.\n",
-    "vertical sparse": "...*....\n" "...*....\n" "...*....\n" "...*....\n",
-    "vertical sparse, thick": "...**...\n" "...**...\n" "...**...\n" "...**...\n",
-    "horizontal dense": "**\n" "..\n",
-    "horizontal": "....\n" "****\n" "....\n" "....\n",
-    "horizontal thick": "....\n" "****\n" "****\n" "....\n",
+    "vertical dense": "*.\n*.\n",
+    "vertical": ".*..\n.*..\n.*..\n.*..\n",
+    "vertical thick": ".**.\n.**.\n.**.\n.**.\n",
+    "vertical sparse": "...*....\n...*....\n...*....\n...*....\n",
+    "vertical sparse, thick": "...**...\n...**...\n...**...\n...**...\n",
+    "horizontal dense": "**\n..\n",
+    "horizontal": "....\n****\n....\n....\n",
+    "horizontal thick": "....\n****\n****\n....\n",
     "horizontal sparse": "........\n"
     "........\n"
     "........\n"
@@ -260,9 +257,9 @@ _klayout_dither_patterns = {
     "........\n"
     "........\n"
     "........\n",
-    "grid dense": "**\n" "*.\n",
-    "grid": ".*..\n" "****\n" ".*..\n" ".*..\n",
-    "grid thick": ".**.\n" "****\n" "****\n" ".**.\n",
+    "grid dense": "**\n*.\n",
+    "grid": ".*..\n****\n.*..\n.*..\n",
+    "grid thick": ".**.\n****\n****\n.**.\n",
     "grid sparse": "...*....\n"
     "...*....\n"
     "...*....\n"
@@ -297,14 +294,12 @@ class HatchPattern(BaseModel):
 
     @field_validator("custom_pattern")
     @classmethod
-    def check_pattern_klayout(cls, pattern: str | None, **kwargs: Any) -> str | None:
+    def check_pattern_klayout(cls, pattern: str | None) -> str | None:
         if pattern is None:
             return None
-        lines = pattern.splitlines()
-        if any(len(list(line)) > 32 for line in lines):
-            raise ValueError(
-                f"Custom pattern {kwargs['values']['name']} has more than 32 characters."
-            )
+        # Optimization: Use len(line) directly without converting to list
+        if any(len(line) > 32 for line in pattern.splitlines()):
+            raise ValueError(f"Custom pattern {pattern} has more than 32 characters.")
         return pattern
 
     def to_klayout_xml(self) -> ET.Element:
@@ -343,18 +338,23 @@ class LineStyle(BaseModel):
 
     @field_validator("custom_style")
     @classmethod
-    def check_pattern(cls, pattern: str | None, **kwargs: Any) -> str | None:
+    def check_pattern(cls, pattern: str | None) -> str | None:
         if pattern is None:
             return None
 
-        pattern_list = list(pattern)
-        valid_chars = all(char in ["*", "."] for char in pattern_list)
-        valid_length = len(pattern_list) <= 32
-        if (not valid_chars) or (not valid_length):
+        # Check length first (it's faster)
+        if len(pattern) > 32:
             raise ValueError(
-                f"Custom line pattern {kwargs['values']['name']} must consist of '*' and '.' characters and be no more than 32 characters long."
+                f"Custom line pattern {pattern} must consist of '*' and '.' characters and be no more than 32 characters long."
             )
 
+        valid_chars = {"*", "."}
+        for char in pattern:
+            if char not in valid_chars:
+                raise ValueError(
+                    f"Custom line pattern {pattern} must consist of '*' and '.' characters and be no more than 32 characters long."
+                )
+
         return pattern
 
     def to_klayout_xml(self) -> ET.Element:
@@ -422,7 +422,7 @@ class LayerView(BaseModel):
     marked: bool = False
     xfill: bool = False
     animation: int = 0
-    group_members: builtins.dict[str, LayerView] | None = Field(default={})
+    group_members: builtins.dict[str, LayerView] = Field(default_factory=builtins.dict)
 
     def __init__(
         self,
@@ -455,23 +455,17 @@ class LayerView(BaseModel):
 
         super().__init__(**data)
 
-        # Iterate through all items, adding group members as needed
-        for name, field in self.model_fields.items():
-            default = field.get_default()
-            if isinstance(default, LayerView):
-                self.group_members[name] = default
-
     def dict(
         self,
         *,
-        include: AbstractSetIntStr | MappingIntStrAny | None = None,
-        exclude: AbstractSetIntStr | MappingIntStrAny | None = None,
+        include: IncEx | None = None,
+        exclude: IncEx | None = None,
         by_alias: bool = False,
         exclude_unset: bool = False,
         exclude_defaults: bool = False,
         exclude_none: bool = False,
         simplify: bool = True,
-    ) -> DictStrAny:
+    ) -> dict[str, Any]:
         """Generate a dictionary representation of the model, optionally specifying which fields to include or exclude.
 
         Specify "simplify" to consolidate fill and frame color/brightness if they are the same.
@@ -538,8 +532,8 @@ class LayerView(BaseModel):
         else:
             return 0.3
 
-    def get_color_dict(self) -> dict[str, str]:
-        if self.fill_color is not None or self.frame_color is not None:
+    def get_color_dict(self) -> builtins.dict[str, str | None]:
+        if self.fill_color is not None and self.frame_color is not None:
             return {
                 "fill_color": ensure_six_digit_hex_color(self.fill_color.as_hex()),
                 "frame_color": ensure_six_digit_hex_color(self.frame_color.as_hex()),
@@ -559,11 +553,18 @@ class LayerView(BaseModel):
             "#3d87cc",
             "#e5520e",
         ]
-        color = layer_colors[np.mod(self.layer[0], len(layer_colors))]
+        if self.layer is not None:
+            color = layer_colors[int(np.mod(self.layer[0], len(layer_colors)))]
+        else:
+            color = None
         return {"fill_color": color, "frame_color": color}
 
     def _build_klayout_xml_element(
-        self, tag: str, name: str, custom_hatch_patterns: dict, custom_line_styles: dict
+        self,
+        tag: str,
+        name: str,
+        custom_hatch_patterns: builtins.dict[str, HatchPattern],
+        custom_line_styles: builtins.dict[str, LineStyle],
     ) -> ET.Element:
         """Get XML Element from attributes."""
         # If hatch pattern name matches a named (built-in) KLayout pattern, use 'I<idx>' notation
@@ -571,28 +572,30 @@ class LayerView(BaseModel):
         if hatch_name is None:
             dither_pattern = None
         elif hatch_name in _klayout_dither_patterns:
-            dither_pattern = f"I{list(_klayout_dither_patterns).index(hatch_name)}"
+            dither_pattern = f"I{list(_klayout_dither_patterns).index(str(hatch_name))}"
         elif hatch_name in custom_hatch_patterns:
-            dither_pattern = f"C{list(custom_hatch_patterns).index(hatch_name)}"
+            dither_pattern = f"C{list(custom_hatch_patterns).index(str(hatch_name))}"
         else:
             warnings.warn(
-                f"Dither pattern {hatch_name!r} does not correspond to any KLayout built-in or custom pattern! Using 'solid' instead."
+                f"Dither pattern {hatch_name!r} does not correspond to any KLayout built-in or custom pattern! Using 'I3' instead.",
+                stacklevel=3,
             )
-            dither_pattern = "solid"
+            dither_pattern = "I3"
 
         # If line style name matches a named (built-in) KLayout pattern, use 'I<idx>' notation
         ls_name = getattr(self.line_style, "name", self.line_style)
         if ls_name is None:
             line_style = None
         elif ls_name in _klayout_line_styles:
-            line_style = f"I{list(_klayout_line_styles).index(ls_name)}"
+            line_style = f"I{list(_klayout_line_styles).index(str(ls_name))}"
         elif ls_name in custom_line_styles:
-            line_style = f"C{list(custom_line_styles).index(ls_name)}"
+            line_style = f"C{list(custom_line_styles).index(str(ls_name))}"
         else:
             warnings.warn(
-                f"Line style {ls_name!r} does not correspond to any KLayout built-in or custom pattern! Using 'solid' instead."
+                f"Line style {ls_name!r} does not correspond to any KLayout built-in or custom pattern! Using 'I3' instead.",
+                stacklevel=3,
             )
-            line_style = "solid"
+            line_style = "I3"
 
         frame_color = (
             ensure_six_digit_hex_color(self.frame_color.as_hex())
@@ -618,12 +621,16 @@ class LayerView(BaseModel):
             "marked": str(self.marked).lower(),
             "xfill": str(self.xfill).lower(),
             "animation": self.animation,
-            "name": f"{name} {self.layer[0]}/{self.layer[1]}"
-            if self.layer_in_name
-            else name,
-            "source": f"{self.layer[0]}/{self.layer[1]}@1"
-            if self.layer is not None
-            else "*/*@*",
+            "name": (
+                f"{name} {self.layer[0]}/{self.layer[1]}"
+                if self.layer_in_name and self.layer is not None
+                else name
+            ),
+            "source": (
+                f"{self.layer[0]}/{self.layer[1]}@1"
+                if self.layer is not None
+                else "*/*@*"
+            ),
         }
         el = ET.Element(tag)
         for key, value in prop_dict.items():
@@ -639,12 +646,14 @@ class LayerView(BaseModel):
         return el
 
     def to_klayout_xml(
-        self, custom_hatch_patterns: dict, custom_line_styles: dict
+        self,
+        custom_hatch_patterns: builtins.dict[str, HatchPattern],
+        custom_line_styles: builtins.dict[str, LineStyle],
     ) -> ET.Element:
         """Return an XML representation of the LayerView."""
         props = self._build_klayout_xml_element(
             "properties",
-            name=self.name,
+            name=self.name or "",
             custom_hatch_patterns=custom_hatch_patterns,
             custom_line_styles=custom_line_styles,
         )
@@ -661,7 +670,7 @@ class LayerView(BaseModel):
 
     @staticmethod
     def _process_name(
-        name: str, layer_pattern: str | re.Pattern
+        name: str, layer_pattern: str | re.Pattern[str]
     ) -> tuple[str | None, bool | None]:
         """Strip layer info from name if it exists.
 
@@ -680,7 +689,9 @@ class LayerView(BaseModel):
         return clean_name(name, remove_dots=True), layer_in_name
 
     @staticmethod
-    def _process_layer(layer: str, layer_pattern: str | re.Pattern) -> Layer | None:
+    def _process_layer(
+        layer: str, layer_pattern: str | re.Pattern[str]
+    ) -> Layer | None:
         """Convert .lyp XML layer entry to a Layer.
 
         Args:
@@ -695,7 +706,7 @@ class LayerView(BaseModel):
 
     @classmethod
     def from_xml_element(
-        cls, element: ET.Element, layer_pattern: str | re.Pattern
+        cls, element: ET.Element, layer_pattern: str | re.Pattern[str]
     ) -> LayerView | None:
         """Read properties from .lyp XML and generate LayerViews from them.
 
@@ -703,13 +714,15 @@ class LayerView(BaseModel):
             element: XML Element to iterate over.
             layer_pattern: Regex pattern to match layers with.
         """
+        element_name = element.find("name")
         name, layer_in_name = cls._process_name(
-            element.find("name").text, layer_pattern
+            element_name.text or "" if element_name is not None else "",
+            layer_pattern,
         )
         if name is None:
             return None
 
-        hatch_pattern = element.find("dither-pattern").text
+        hatch_pattern = element.find("dither-pattern").text  # type: ignore[union-attr]
         # Translate KLayout index to hatch name
         if hatch_pattern and re.match(r"I\d+", hatch_pattern):
             hatch_pattern = list(_klayout_dither_patterns.keys())[
@@ -718,18 +731,24 @@ class LayerView(BaseModel):
 
         # Translate KLayout index to line style name
         line_style = element.find("line-style")
-        if line_style and re.match(r"I\d+", line_style.text):
-            line_style = list(_klayout_line_styles.keys())[int(line_style.text[1:])]
+        if (
+            line_style is not None
+            and line_style.text is not None
+            and re.match(r"I\d+", line_style.text)
+        ):
+            line_style = list(_klayout_line_styles.keys())[int(line_style.text[1:])]  # type: ignore[assignment]
 
         lv = LayerView(
             name=name,
-            layer=cls._process_layer(element.find("source").text, layer_pattern),
+            layer=cls._process_layer(element.find("source").text, layer_pattern),  # type: ignore[union-attr,arg-type]
             fill_color=getattr(element.find("fill-color"), "text", None),
             frame_color=getattr(element.find("frame-color"), "text", None),
-            fill_brightness=element.find("fill-brightness").text or 0,
-            frame_brightness=element.find("frame-brightness").text or 0,
+            fill_brightness=element.find("fill-brightness").text or 0,  # type: ignore[union-attr]
+            frame_brightness=element.find("frame-brightness").text or 0,  # type: ignore[union-attr]
             hatch_pattern=hatch_pattern or None,
-            line_style=line_style or None,
+            line_style=line_style
+            if line_style is not None and len(line_style) > 0
+            else None,
             valid=getattr(element.find("valid"), "text", True),
             visible=getattr(element.find("visible"), "text", True),
             transparent=getattr(element.find("transparent"), "text", False),
@@ -741,10 +760,10 @@ class LayerView(BaseModel):
         )
 
         # Add only if needed, so we can filter by defaults when dumping to yaml
-        group_members = {}
+        group_members: builtins.dict[str, LayerView] = {}
         for member in element.iterfind("group-members"):
             member_lv = cls.from_xml_element(member, layer_pattern)
-            if member_lv:
+            if member_lv and member_lv.name is not None:
                 group_members[member_lv.name] = member_lv
 
         if group_members != {}:
@@ -799,19 +818,24 @@ class LayerViews(BaseModel):
             data["layer_views"] = lvs.layer_views
             data["custom_line_styles"] = lvs.custom_line_styles
             data["custom_dither_patterns"] = lvs.custom_dither_patterns
-
+        layer_names: builtins.dict[str, LayerEnum] | None = None
         if layers:
-            layers = {layer.name: layer for layer in layers if layer is not None}
+            layer_names = {layer.name: layer for layer in layers if layer is not None}  # type: ignore[attr-defined]
+        else:
+            layer_names = None
 
         super().__init__(**data)
 
         for name in self.model_dump():
             lv = getattr(self, name)
             if isinstance(lv, LayerView):
-                #
-                if layers is not None and name in layers:
+                if (
+                    layers is not None
+                    and layer_names is not None
+                    and name in layer_names
+                ):
                     lv_dict = lv.dict(exclude={"layer", "name"})
-                    lv = LayerView(layer=layers[name], name=name, **lv_dict)
+                    lv = LayerView(layer=layer_names[name], name=name, **lv_dict)
                 self.add_layer_view(name=name, layer_view=lv)
 
     def add_layer_view(
@@ -836,9 +860,9 @@ class LayerViews(BaseModel):
 
         # If the dither pattern is a CustomDitherPattern, add it to custom_patterns
         dither_pattern = layer_view.hatch_pattern
-        if (
-            isinstance(dither_pattern, HatchPattern)
-            and dither_pattern not in self.custom_dither_patterns.keys()
+        if isinstance(dither_pattern, HatchPattern) and (
+            dither_pattern.name is not None
+            and dither_pattern.name not in self.custom_dither_patterns.keys()
         ):
             self.custom_dither_patterns[dither_pattern.name] = dither_pattern
 
@@ -850,9 +874,9 @@ class LayerViews(BaseModel):
             layer_view.hatch_pattern = self.custom_dither_patterns[dither_pattern]
 
         line_style = layer_view.line_style
-        if (
-            isinstance(line_style, LineStyle)
-            and line_style not in self.custom_line_styles.keys()
+        if isinstance(line_style, LineStyle) and (
+            line_style.name is not None
+            and line_style.name not in self.custom_line_styles.keys()
         ):
             self.custom_line_styles[line_style.name] = line_style
         elif (
@@ -924,7 +948,6 @@ class LayerViews(BaseModel):
         Returns:
             LayerView.
         """
-        layer_tuple = tuple(layer_tuple)
         tuple_to_name = {v.layer: k for k, v in self.get_layer_views().items()}
         if layer_tuple not in tuple_to_name:
             raise ValueError(
@@ -936,7 +959,11 @@ class LayerViews(BaseModel):
 
     def get_layer_tuples(self) -> set[Layer]:
         """Returns a tuple for each layer."""
-        return {layer.layer for layer in self.get_layer_views().values()}
+        return {
+            layer.layer
+            for layer in self.get_layer_views().values()
+            if layer.layer is not None
+        }
 
     def clear(self) -> None:
         """Deletes all layers in the LayerViews."""
@@ -962,15 +989,22 @@ class LayerViews(BaseModel):
 
         non_empty_layers = [v for v in layer_views.values() if v.layer is not None]
 
-        sorted_layers = sorted(non_empty_layers, key=lambda x: (x.layer[0], x.layer[1]))
+        sorted_layers = sorted(
+            non_empty_layers,
+            key=lambda x: (x.layer[0], x.layer[1]),  # type: ignore[index]
+        )
 
         for n, layer in enumerate(sorted_layers):
             layer_tuple = layer.layer
+            if layer_tuple is None:
+                continue
             rectangle = gf.components.rectangle(
                 size=(100 * scale, 100 * scale), layer=layer_tuple
             )
             text = gf.components.text(
-                text=f"{layer.name}\n{layer_tuple[0]} / {layer_tuple[1]}",
+                text=f"{layer.name}\n{layer_tuple[0]} / {layer_tuple[1]}"
+                if layer_tuple is not None
+                else layer.name,
                 size=20 * scale,
                 position=(50 * scale, -20 * scale),
                 justify="center",
@@ -980,11 +1014,11 @@ class LayerViews(BaseModel):
             xloc = n % matrix_size
             yloc = int(n // matrix_size)
             ref = component.add_ref(rectangle)
-            ref.dmovex((100 + spacing) * xloc * scale)
-            ref.dmovey(-(100 + spacing) * yloc * scale)
+            ref.movex((100 + spacing) * xloc * scale)
+            ref.movey(-(100 + spacing) * yloc * scale)
             ref = component.add_ref(text)
-            ref.dmovex((100 + spacing) * xloc * scale)
-            ref.dmovey(-(100 + spacing) * yloc * scale)
+            ref.movex((100 + spacing) * xloc * scale)
+            ref.movey(-(100 + spacing) * yloc * scale)
         return component
 
     def to_lyp(
@@ -1025,7 +1059,7 @@ class LayerViews(BaseModel):
     @staticmethod
     def from_lyp(
         filepath: str | pathlib.Path,
-        layer_pattern: str | re.Pattern | None = None,
+        layer_pattern: str | re.Pattern[str] | None = None,
     ) -> LayerViews:
         r"""Write all layer properties to a KLayout .lyp file.
 
@@ -1037,28 +1071,31 @@ class LayerViews(BaseModel):
 
         filepath = pathlib.Path(filepath)
 
-        if not os.path.exists(filepath):
-            raise OSError(f"File {str(filepath)!r} does not exist, cannot read.")
+        if not filepath.exists():
+            raise FileNotFoundError(
+                f"File {str(filepath)!r} does not exist, cannot read."
+            )
 
         tree = ET.parse(filepath)
         root = tree.getroot()
         if root.tag != "layer-properties":
             raise OSError("Layer properties file incorrectly formatted, cannot read.")
 
-        dither_patterns = {}
+        dither_patterns: dict[str, HatchPattern] = {}
         for dither_block in root.iter("custom-dither-pattern"):
-            name = dither_block.find("name").text
-            order = dither_block.find("order").text
+            name = dither_block.find("name").text  # type: ignore[union-attr]
+            order = dither_block.find("order").text  # type: ignore[union-attr]
 
             if name is None or order is None:
                 continue
             pattern = "\n".join(
-                [line.text for line in dither_block.find("pattern").iter()]
+                [line.text for line in dither_block.find("pattern").iter()]  # type: ignore[misc,union-attr]
             )
 
             if name in dither_patterns:
                 warnings.warn(
-                    f"Dither pattern named {name!r} already exists. Keeping only the first defined."
+                    f"Dither pattern named {name!r} already exists. Keeping only the first defined.",
+                    stacklevel=3,
                 )
                 continue
 
@@ -1067,24 +1104,25 @@ class LayerViews(BaseModel):
                 order=int(order),
                 custom_pattern=pattern.lstrip(),
             )
-        line_styles = {}
+        line_styles: dict[str, LineStyle] = {}
         for line_block in root.iter("custom-line-style"):
-            name = line_block.find("name").text
-            order = line_block.find("order").text
+            name = line_block.find("name").text  # type: ignore[union-attr]
+            order = line_block.find("order").text  # type: ignore[union-attr]
 
             if name is None or order is None:
                 continue
 
             if name in line_styles:
                 warnings.warn(
-                    f"Line style named {name!r} already exists. Keeping only the first defined."
+                    f"Line style named {name!r} already exists. Keeping only the first defined.",
+                    stacklevel=3,
                 )
                 continue
 
             line_styles[name] = LineStyle(
                 name=name,
                 order=int(order),
-                custom_style=line_block.find("pattern").text,
+                custom_style=line_block.find("pattern").text,  # type: ignore[union-attr]
             )
 
         layer_views = {}
@@ -1101,23 +1139,16 @@ class LayerViews(BaseModel):
             custom_line_styles=line_styles,
         )
 
-    def to_yaml(
-        self, layer_file: str | pathlib.Path, prefer_named_color: bool = True
-    ) -> None:
+    def to_yaml(self, layer_file: str | pathlib.Path) -> None:
         """Export layer properties to a YAML file.
 
         Args:
             layer_file: Name of the file to write LayerViews to.
-            prefer_named_color: Write the name of a color instead of its hex representation when possible.
         """
         lf_path = pathlib.Path(layer_file)
         dirpath = lf_path.parent
         dirpath.mkdir(exist_ok=True, parents=True)
 
-        add_tuple_yaml_presenter()
-        add_multiline_str_yaml_presenter()
-        add_color_yaml_presenter(prefer_named_color=prefer_named_color)
-
         lvs = {
             name: lv.dict(exclude_none=True, exclude_defaults=True, exclude_unset=True)
             for name, lv in self.layer_views.items()
@@ -1145,6 +1176,7 @@ class LayerViews(BaseModel):
                 sort_keys=False,
                 default_flow_style=False,
                 encoding="utf-8",
+                Dumper=TechnologyDumper,
             )
         )
 
@@ -1157,7 +1189,7 @@ class LayerViews(BaseModel):
         """
         layer_file = pathlib.Path(layer_file)
 
-        properties = yaml.safe_load(layer_file.open())
+        properties = yaml.safe_load(layer_file.read_text())
         lvs = {}
         for name, lv in properties["LayerViews"].items():
             if "group_members" in lv:
@@ -1208,6 +1240,7 @@ if __name__ == "__main__":
     PDK = get_generic_pdk()
     LAYER_VIEWS = PDK.layer_views
     # LAYER_VIEWS.to_yaml(PATH.repo / "extra" / "layers.yml")
+    assert LAYER_VIEWS is not None
     c = LAYER_VIEWS.preview_layerset()
     c.show()
 
